#!/bin/bash

# Redirect stdout to system.log
exec > /home/tasc/crest/log/system.log
exec 3>&1

# Redirect stderr to stdout  (2>&1)
exec 4>&2 2>&1

echo "============================================================================================"
echo "Summit - System Initialization"
echo "  Version 1.1.0 - October 30, 2018"
echo "  (c) TASC Systems Inc."
echo "--------------------------------------------------------------------------------------------"
date
echo

# Confirm that the user has privilege to run this setup script
echo "Checking user privilige"
if [ $EUID -ne 0 ]
  then echo "   Exiting! Please re-run as priviliged user"
  exit
fi
echo "   OK, running with sufficient privileges"
echo

# Read system configuration from Summit config file
echo "Reading Summit.config file to determine configuration requirements ..."
source /home/tasc/crest/summit.config
echo "   Digital Boards installed     : $digitalboards"
echo "   Digital Inputs  Board 1 base : $digitalinputs1location"
echo "   Digital Inputs  Board 1 count: $digitalinputs1"
echo "   Digital Output  Board 1 base : $digitaloutputs1location"
echo "   Digital Outputs Board 1 count: $digitaloutputs1"
echo
echo "   Analog Boards installed     : $analogboards"
echo "   Analog Inputs chips / board : $analoginputchipsperboard"
echo "   Analog Inputs / chip        : $analoginputsperchip"
echo "   Analog Inputs reference     : $analoginputreference"
echo "   Analog Inputs configuration : $analoginputconfiguration"
echo "   Analog Outputs / board      : $analogoutputsperboard"
echo "   Analog Output measure method: $analogoutputreferencemethod"
echo "   Analog Outputs enable       : $analogoutputenable"
echo
echo "   Default LED GPIO            : $defaultLedGpioLocation"
echo

# Processing LED GPIO
echo "Processing LED GPIO"
cd /sys/class/gpio
CWD=$(pwd)
echo "  Changing to $CWD to process LED GPIO"
echo $defaultLedGpioLocation > export
cd $CWD/gpio$defaultLedGpioLocation
echo out > $CWD/gpio$defaultLedGpioLocation/direction
echo 1 > $CWD/gpio$defaultLedGpioLocation/value

echo "LED to blue blinking state to indicate boot up progress"
cd /sys/class/leds
CWD=$(pwd)
echo none > $CWD/led:red/trigger
echo none > $CWD/led:green/trigger
echo none > $CWD/led:blue/trigger
echo 0 > $CWD/led:red/brightness
echo 1 > $CWD/led:green/brightness
echo 0 > $CWD/led:blue/brightness
echo

# Processing Analog Boards
echo "Processing Analog Boards"
cd /sys/class/hwmon
CWD=$(pwd)
echo "  Changing to $CWD to process Analog Boards"
BOARDCOUNTER=0
while [ $BOARDCOUNTER -lt $analogboards ]; do
	echo "  Configuring Analog Board $BOARDCOUNTER"

	echo "    Configuring Analog Inputs"
	# Each board has multiple chips
	CHIPCOUNTER=0
	while [ $CHIPCOUNTER -lt $analoginputchipsperboard ]; do
		# The location of the chip we are going to configure
		ANALOGINPUTDIR="./hwmon$CHIPCOUNTER/"

		# Set the voltage reference for each ADC chip
		#	Default reference = 5000
		ANALOGINPUTVREFFILE='chip_vref'
		ANALOGINPUTVREFLOC="$ANALOGINPUTDIR$ANALOGINPUTVREFFILE"
		echo "    Setting reference for analog input chip at $ANALOGINPUTVREFLOC (Reference Value=$analoginputreference)"
		echo $analoginputreference > $ANALOGINPUTVREFLOC

		IOCOUNTER=0
		while [ $IOCOUNTER -lt $analoginputsperchip ]; do
  			# Configure the input points for this chip
			#	0xC4 =
			#		0x80 - 1=enabled, 0=disabled
			#		0x40 - 1=single-ended, 0=differential
			#		0x04 - LTC244X_CHAN_OSR_512
			ANALOGINPUTCONFIGFILE="in"$IOCOUNTER"_conf"
			ANALOGINPUTCONFIGLOC="$ANALOGINPUTDIR$ANALOGINPUTCONFIGFILE"
			echo "    Configuring analog input chip at $ANALOGINPUTCONFIGLOC (Configuration Method=$analoginputconfiguration)"
			echo $analoginputconfiguration > $ANALOGINPUTCONFIGLOC

			let IOCOUNTER=IOCOUNTER+1
			sleep 0.1
		done

		let CHIPCOUNTER=CHIPCOUNTER+1
	done

	echo "    Configuring Analog Outputs"
	CHIPCONFDIR="./hwmon2/chip_conf"
	echo "    Configuring analog output chip at $CHIPCONFDIR (Filter Method=$analogoutputreferencemethod)"
	echo $analogoutputreferencemethod > $CHIPCONFDIR

	IOCOUNTER=0
	while [ $IOCOUNTER -lt $analogoutputsperboard ]; do
		ANALOGOUTPUTCONFDIR1="./hwmon2/out"
		ANALOGOUTPUTCONFDIR2="$ANALOGOUTPUTCONFDIR1$IOCOUNTER"
		ANALOGOUTPUTCONFDIR=$ANALOGOUTPUTCONFDIR2"_conf"
		echo "      Configuring analog output at $ANALOGOUTPUTCONFDIR (Measure Method=$analogoutputenable)"
		echo $analogoutputenable > $ANALOGOUTPUTCONFDIR
		let IOCOUNTER=IOCOUNTER+1
		sleep 0.1
	done

	let BOARDCOUNTER=BOARDCOUNTER+1
done
echo

# Processing Digital Boards
echo "Processing Digital Boards"
cd /sys/class/gpio
CWD=$(pwd)
echo "  Changing to $CWD to process Digital Boards"
BOARDCOUNTER=0
while [ $BOARDCOUNTER -lt $digitalboards ]; do
	echo "  Configuring Digital Board $BOARDCOUNTER"

	# Processing Digital Inputs
	echo "  Processing Digital Inputs"
    IOBASE=$digitalinputs1location

	IOCOUNTER=0
	while [ $IOCOUNTER -lt $digitalinputs1 ]; do
		let IOLOCATION=$IOBASE+$IOCOUNTER
		echo "      Configuring Digital Input "$IOCOUNTER" to "$IOLOCATION
		echo $IOLOCATION > export

		let IOCOUNTER=IOCOUNTER+1
		sleep 0.1
	done

	# Processing Digital Outputs
	echo "  Processing Digital Outputs"
	IOCOUNTER=0
	IOBASE=$digitaloutputs1location
	while [ $IOCOUNTER -lt $digitaloutputs1 ]; do
		echo "      Configuring Digital Output $IOCOUNTER"
		let IOLOCATION=$IOBASE+$IOCOUNTER
		echo $IOLOCATION > export
		let IOCOUNTER=IOCOUNTER+1
		sleep 0.1
	done

	let BOARDCOUNTER=BOARDCOUNTER+1
done

# Start node app.js
echo
echo
echo "----- Starting Summit Administration Application -----"
cd /home/tasc/crest
CWD=$(pwd)

# Ensure that the log directory is there
mkdir log

# Turn off redirection, to allow forever app write to the system.log file
#exec 1>&3 3>&-
#exec 2>&4 4>&-

# Setup the forever and node directories
NODE_BIN_DIR=/usr/local/bin
NODE_PATH=/home/tasc/crest/node_modules/
APPLICATION_DIRECTORY=/home/tasc/crest
APPLICATION_START=app.js
PIDFILE=/var/run/$NAME.pid
LOGFILE=/var/log/$NAME.log

PATH=$NODE_BIN_DIR:$PATH
export NODE_PATH=$NODE_PATH

# Start the Summit Administration Application ...
#/usr/local/bin/forever -v -w --pidFile SummitApp --watchIgnore '{log,data,node_modules}' --sourceDir $APPLICATION_DIRECTORY start $APPLICATION_START
/usr/local/bin/node app.js
echo "Exited Crest node app"
date

echo "LED to red to indicate error"
cd /sys/class/leds
CWD=$(pwd)
echo none > $CWD/led:red/trigger
echo none > $CWD/led:green/trigger
echo none > $CWD/led:blue/trigger
echo 1 > $CWD/led:red/brightness
echo 0 > $CWD/led:green/brightness
echo 0 > $CWD/led:blue/brightness
echo
